function [sspace Nd DopStep codeDl] = DftParallelFrequencyAcquisition( y, loc, Cs, Nd, DopStep, fs, fi )

%%
% Summary:
%   Function that computes the acquisition search space exploiting the FFT
%   algorithm. Parallel search is performed in the frequency domain.
%
% Arguments:
%   y:      [vector] the input GPS samples
%   loc:    [vector] the local code, resampled at the right frequency
%   Cs:     [scalar] downsampling factor for the code search
%   Nd:     [scalar] number of Doppler bins
%   DopStep:[scalar] Doppler step in Hz
%   fs:     [scalar] sampling frequency in Hz
%   fi:     [scalar] intermediate frequency in Hz
%
% Returns:
%   sspace: [matrix] the computed cross-ambiguity function
%   Nd;     [scalar] the effective number of Doppler frequencies used for
%           the computation of the cross-ambiguity function
%   DopStep:[scalar] the effective Doppler step
%   codeDl :[vector] the code delays used for the computation
%
% History:
%   Aug 05/12 - Function created by Daniele Borio.
%%

% 1) Determine the parameters for the computation
% Code delays in samples
Nc = length( y );       % Code length
codeDl = 0:Cs:( Nc - 1 );

% Doppler parameters
fmax = round( ( Nd - 1 ) / 2 ) * DopStep;   % Maximum Doppler frequency to search
Cd = floor( fs / fmax / 2 );                % Down sampling factor

fmax = fs / 2 / Cd;                         % New maximum Doppler frequency accounting
                                            % for the 'floor' operator

if bitand( Nd, 1 ) == 0,                    % It is an even number                                             
    Nd = 2 * ceil( fmax / DopStep );        % Number of Dopppler frequencies
else
    Nd = 2 * ceil( fmax / DopStep ) + 1;
end

Nd = 2^( ceil( log2( Nd ) ) );              % Make it a power of 2 for efficiency

DopStep = 2 * fmax / Nd;                    % Actual Doppler step 

% IF removal
ybase = y .* exp( -1i * 2 * pi * (0:(Nc -1)) * fi / fs );

% Finally compute the search space
sspace = zeros( Nd, length( codeDl ) );

for ii = 1:length(codeDl),
    % wipe-off the code
    yc = ybase .* [ loc((codeDl(ii) + 1):end) loc(1:codeDl(ii)) ];
    
    % integrate and dump
    Ns = floor( length( yc ) / Cd );
    yid = mean( reshape( yc(1:(Cd * Ns)), Cd, Ns ), 1 );
    
    % compute the FFT
    sspace( :, ii ) = fftshift( fft( yid, Nd ) );
    
end

% Compute the square modulus
sspace = real( sspace .* conj( sspace ) );

% Convert the code Delay in seconds
codeDl = codeDl / fs;
